import unittest

from iutest.core.runners import runnerconstants
from iutest.core.runners import pyunitrunner
from iutest.tests.iutests import test_runnercommon as common


class PyUnitRunnerTestCase(unittest.TestCase):
    def setUp(self):
        common.setUpTest(self, runnerconstants.RUNNER_PYUNIT)

    def test_parseParameterizedTestId(self):
        common.checkParseParameterizedTestId(self)

    def test_isValid(self):
        self.assertTrue(pyunitrunner.PyUnitRunner.isValid())

    def test_listByDir(self):
        common.checkListByDir(self)

    def test_listByModulePath(self):
        common.checkListByModulePath(self)

    def test_runTests(self):
        common.checkRunTests(self)

    def test_partialRun(self):
        common.checkPartialRun(self)

    def test_lastRunInfo(self):
        common.checkLastRunInfo(self)



import os
from iutest.core import pathutils
from iutest.qt import iconFromPath

def _iconDir():
    return os.path.join(pathutils.iutestPackageDir(), "icons")


def iconPath(iconName):
    return os.path.join(_iconDir(), iconName)


def iconPathSet(iconName, suffixes):
    iconDir = _iconDir()
    nameParts = list(iconName.partition("."))
    nameParts.insert(1, None)
    paths = []
    for suffix in suffixes:
        nameParts[1] = suffix
        fileName = "".join(nameParts)
        paths.append(os.path.join(iconDir, fileName))

    return paths


def initSingleClassIcon(obj, objAttributeName, iconFileName):
    path = iconPath(iconFileName)
    setattr(obj, objAttributeName, iconFromPath(path))


import unittest

from iutest.core import appsettings


class AppSettingsTestCase(unittest.TestCase):
    def test_saveRestore(self):
        setting = appsettings.AppSettings.get()
        data = {
            "testInt":(1, 1),
            "testInt1":(True, 1),
            "testInt2":("1", 1),
            "testBool":(True, True),
            "testBool1":(1, True),
            "testBool2":("1", True),
            "testStr":("str", "str"),
            "testStr1":(1, "1"),
            "testStr2":("", ""),
        }
        for key, (inputValue, _) in data.items():
            setting.saveSimpleConfig(key, inputValue, sync=False)

        for key, (inputValue, expectedValue) in data.items():
            self.assertEqual(setting.simpleConfigValue(key), inputValue)

            if "Int" in key:
                self.assertEqual(setting.simpleConfigIntValue(key), expectedValue)
                
            elif "Bool" in key:
                self.assertEqual(setting.simpleConfigBoolValue(key), expectedValue)

            elif "Str" in key:
                self.assertEqual(setting.simpleConfigStrValue(key), expectedValue)

        for key in data:
            setting.removeConfig(key)
            self.assertTrue(setting.simpleConfigValue(key) is None)



RUNNER_PYUNIT = 0
RUNNER_NOSE2 = 1
RUNNER_PYTEST = 2

RUNNER_NAMES = {RUNNER_PYUNIT: "PyUnit", RUNNER_NOSE2: "nose2", RUNNER_PYTEST: "pytest"}


def isValidRunnerName(runnerName):
    return runnerName and runnerName in RUNNER_NAMES.values()


def runnerModeFromName(runnerName):
    for mode, name in RUNNER_NAMES.items():
        if runnerName == name:
            return mode

    return RUNNER_PYUNIT



import os
import inspect
import logging

logger = logging.getLogger(__name__)


def iutestPackageDir():
    filePath = os.path.abspath(inspect.getfile(inspect.currentframe()))
    return os.path.dirname(os.path.dirname(filePath))


def iutestRootDir():
    return os.path.dirname(iutestPackageDir())


def isPath(path):
    return "/" in path or "\\" in path


def objectFromDotPath(dotPath, silent=False):
    def tryImportClosestModule(paths):
        module = None
        while paths:
            try:
                module = __import__(".".join(paths))
                break
            except:
                del paths[-1]
                if not paths:
                    return None
        return module

    parts = dotPath.split(".")
    module = tryImportClosestModule(parts[:])
    if not module:
        if not silent:
            logger.error("No module found from %s", dotPath)
        return None

    # Now get the object from the module:
    obj = module
    for part in parts[1:]:
        try:
            obj = getattr(obj, part)
        except AttributeError:
            if not silent:
                logger.exception("Error importing the module at path %s", dotPath)
            return None

    return obj


def sourceFileAndLineFromObject(obj):
    sourceFile = inspect.getsourcefile(obj)
    if not sourceFile:
        return (None, None)

    try:
        line = inspect.getsourcelines(obj)[-1]
    except:
        line = 0

    return sourceFile, line


def sourcePathAndLineFromModulePath(dotPath):
    """Get the python file path from a module path.
    Args:
        dotPath (str): the python module path.
    Return:
        str: The python file path.
    """
    try:
        obj = objectFromDotPath(dotPath)
        return sourceFileAndLineFromObject(obj)
    except Exception:
        logger.error("Unable to retrieve source file from %s", dotPath)
    return None, None


import weakref
import re
import sys
import logging
from iutest.core import constants

logger = logging.getLogger(__name__)

class UiStream(object):
    _testMainWindow = None
    _logBrowser = None

    _urlTemplate = "<a href='?{1}={2}'>{0}</a>"
    _traceExp = None

    @classmethod
    def setUi(cls, wgt):
        cls._testMainWindow = weakref.ref(wgt)
        cls._logBrowser = cls.callUiMethod("getLogBrowserWidget")

    @classmethod
    def unsetUi(cls, wgt):
        if cls._testMainWindow and cls._testMainWindow() == wgt:
            cls._testMainWindow = None

    @classmethod
    def logBrowser(cls):
        return cls._logBrowser

    @classmethod
    def _initExp(cls):
        if not cls._traceExp:
            cls._traceExp = re.compile(r"File \"(.*?)\", line (.*?),")

    def __init__(self):
        self._initExp()
        self._testResult = None
        self._linkInfo = None
        self._processStackTraceLink = False
        self._captureStdOut = False
        self._testWindow = None

    def setResult(self, result=None):
        self._testResult = result

    def resultCtx(self, result):
        return _UiStreamSetResultCtx(self, result)

    def setLinkInfo(self, linkInfo=None):
        self._linkInfo = linkInfo

    def linkInfoCtx(self, linkInfo):
        return _UiStreamLinkInfoCtx(self, linkInfo)

    def _processLinkInMsg(self, msg):
        if not self._linkInfo:
            return msg

        html = self._urlTemplate.format(*self._linkInfo)
        return msg.replace(self._linkInfo[0], html)

    def _processLinkInStackTrace(self, msg):
        if not self._processStackTraceLink:
            return msg
            
        lines = msg.split("\n")
        for i, line in enumerate(lines):
            matches = re.finditer(self._traceExp, line)
            if not matches:
                continue

            data = []
            for _, match in enumerate(matches, start=1):
                for groupNum in range(0, len(match.groups())):
                    groupNum = groupNum + 1
                    data.append(match.group(groupNum))

            if not len(data) == 2:
                continue

            html = self._urlTemplate.format(data[0], *data)
            lines[i] = line.replace(data[0], html)
        return "<br>".join(lines)

    def setProcessStackTraceLink(self, process):
        self._processStackTraceLink = process

    def processStackTraceLinkCtx(self):
        return _UiStreamProcessLinkCtx(self)

    def write(self, msg):
        reportUi = self.logBrowser()
        if not reportUi:
            return

        msg = self._processLinkInMsg(msg)
        msg = self._processLinkInStackTrace(msg)

        if self._testResult == constants.TEST_RESULT_ERROR:
            reportUi.logError(msg)

        elif self._testResult == constants.TEST_RESULT_FAIL:
            reportUi.logFailed(msg)

        elif self._testResult == constants.TEST_RESULT_SKIP:
            reportUi.logWarning(msg)

        elif self._testResult == constants.TEST_RESULT_PASS:
            reportUi.logSuccess(msg)

        else:
            reportUi.logInformation(msg)

    def writeln(self, msg=None):
        if msg:
            self.write(msg)
        self.write("<br>")

    def flush(self):
        pass
    
    @classmethod
    def callUiMethod(cls, methodName, *args, **kwargs):
        if not cls._testMainWindow:
            return
        ui = cls._testMainWindow() # deref
        method = getattr(ui, methodName)
        if not method:
            logger.error("%s has no method called %s", ui, methodName)
            return None

        return method(*args, **kwargs)


class _UiStreamProcessLinkCtx(object):
    def __init__(self, uiStream):
        self._uiStream = uiStream

    def __enter__(self):
        self._uiStream.setProcessStackTraceLink(True)

    def __exit__(self, *_, **__):
        self._uiStream.setProcessStackTraceLink(False)


class _UiStreamSetResultCtx(object):
    def __init__(self, uiStream, resultCode):
        self._uiStream = uiStream
        self._resultCode = resultCode

    def __enter__(self):
        self._uiStream.setResult(self._resultCode)

    def __exit__(self, *_, **__):
        self._uiStream.setResult()


class _UiStreamLinkInfoCtx(object):
    def __init__(self, uiStream, linkInfo):
        self._uiStream = uiStream
        self._linkInfo = linkInfo

    def __enter__(self):
        self._uiStream.setLinkInfo(self._linkInfo)

    def __exit__(self, *_, **__):
        self._uiStream.setLinkInfo()


def writePlainTextToUiStream(msg):
    reportUi = UiStream.logBrowser()
    if reportUi:
        reportUi.logInformation(msg)


class BaseCapturer(object):
    def __init__(self, originalStream):
        self._originalStream = originalStream

    def write(self, msg):
        writePlainTextToUiStream(msg)
        self._originalStream.write(msg)

    def flush(self):
        pass


class StdOutCapturer(BaseCapturer):
    def start(self):
        sys.stdout = self

    def stop(self):
        sys.stdout = self._originalStream


class StdErrCapturer(BaseCapturer):
    def start(self):
        sys.stderr = self

    def stop(self):
        sys.stderr = self._originalStream


class LogHandler(logging.Handler):
    def __init__(self, forcePlainOutput=False):
        logging.Handler.__init__(self)
        self._forcePlainOutput = forcePlainOutput
        self._rootLogger = logging.getLogger()

    def emit(self, record):
        reportUi = UiStream.logBrowser()
        msg = self.format(record)
        if self._forcePlainOutput:
            reportUi.logInformation(msg)
            return

        if record.levelno == logging.WARNING:
            reportUi.logWarning(msg)
        elif record.levelno == logging.ERROR:
            reportUi.logFailed(msg)
        else:
            reportUi.logInformation(msg)

    def start(self):
        self._rootLogger.addHandler(self)

    def stop(self):
        self._rootLogger.removeHandler(self)


class LogCaptureContext(object):
    def __init__(self, forcePlainOutput=False):
        self._handler = LogHandler(forcePlainOutput)

    def __enter__(self):
        self._handler.start()

    def __exit__(self, *_, **__):
        self._handler.stop()



import unittest
import logging
import nose2
from iutest.core import pathutils
from iutest.core import pyunitutils

logger = logging.getLogger(__name__)

def iterAllTestPathsFromRootDir(startDirOrModuleName, topDir=None):
    argv = [
        "nose2",
        "--plugin",
        "iutest.plugins.nose2plugins.testlister",
        "--list-tests",
        "--plugin",
        "nose2.plugins.loader.eggdiscovery",
        "--plugin",
        "iutest.plugins.nose2plugins.removeduplicated",
        "--exclude-plugin",
        "nose2.plugins.result",
    ]
    if pathutils.isPath(startDirOrModuleName):
        argv.extend(["-s", startDirOrModuleName, "-t", topDir])
    else:
        argv.append(startDirOrModuleName)

    TestCollector.gotError = False
    nose2.discover(argv=argv, exit=False)
    for tid in TestCollector.iterTestIds():
        yield tid


class TestCollector(nose2.events.Plugin):
    """A nose2 plug to collect the tests without running them.
    """

    gotError = False
    commandLineSwitch = (None, "list-tests", "List test but not running them.")
    _mpmode = False
    _importFailureModule = "ModuleImportFailure"
    _loadTestsFailure = "LoadTestsFailure"
    _errorSeparator = "-" * 80

    _testIds = []

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)
        self._mpmode = True

    @classmethod
    def resetIds(cls):
        cls._testIds = []

    @classmethod
    def collectId(cls, testId):
        if testId not in cls._testIds:
            cls._testIds.append(testId)

    @classmethod
    def iterTestIds(cls):
        for tid in cls._testIds:
            yield tid

    def startTestRun(self, event):
        self.resetIds()
        if self._mpmode:
            return
        event.executeTests = self.collectTests

    def startSubprocess(self, event):
        self.resetIds()
        event.executeTests = self.collectTests
        event.StopTestRunEvent = None

    def _checkError(self, test, testId, mark, msg):
        if test.__class__.__name__ == mark:
            modulename = testId.split(mark)[-1][1:]
            raiser = getattr(test, modulename, None)
            if not raiser:
                return True
            try:
                raiser()
            except Exception:
                logger.exception(msg, modulename)
                return False

        return True

    def collectTests(self, suite, result):
        """Collect tests, but don't run them"""
        for test in suite:
            if isinstance(test, unittest.BaseTestSuite):
                self.collectTests(test, result)
                continue

            testId = test.id()
            # Log the error to trouble shoot the import error:
            if not self._checkError(
                test,
                testId,
                self._importFailureModule,
                "Unable to import the module %s",
            ):
                TestCollector.gotError = True
                continue

            if not self._checkError(
                test,
                testId,
                self._loadTestsFailure,
                "Unable to load tests from %s",
            ):
                TestCollector.gotError = True
                continue

            _, testIdUsed = pyunitutils.parseParameterizedTestId(testId)
            self.collectId(testIdUsed)
